//------------------------------------------------------------------------------
//  Russian Solitaire! v.1.0
//      2006   Steph Stubler
//              University of Virginia
//              CS 216 - Programming & Data Representation [Prof. David Evans]
//
//              Column.cs
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Drawing;

namespace RussianSolitaire
{
    class Column
    {
        ArrayList faceDown, faceUp, stack;
        bool empty;
        int whichCol;

        #region Column Constructors
        /// <summary>
        /// Column 1
        /// </summary>
        /// <param name="u1">Face-up Card 1</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card u1, 
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 1;

            u1.addToStack(true, 1, 0, x, y);
            u1.allowDrop(true);

            faceUp.Add(u1);
            empty = false;
            totalStack();
        }

        /// <summary>
        /// Column 2
        /// </summary>
        /// <param name="d1">Face-Down Card 1</param>
        /// <param name="u1">Face-Up Card 1</param>
        /// <param name="u2">Face-Up Card 2</param>
        /// <param name="u3">Face-Up Card 3</param>
        /// <param name="u4">Face-Up Card 4</param>
        /// <param name="u5">Face-Up Card 5</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card d1, Card u1, Card u2,
            Card u3, Card u4, Card u5, 
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 2;
            d1.flip();

            d1.addToStack(d1.isFaceUp(), 2, 5, x, y);
            y += Constants.YINCFACEDOWN;
            u1.addToStack(u1.isFaceUp(), 2, 4, x, y);
            y += Constants.YINC;
            u2.addToStack(u2.isFaceUp(), 2, 3, x, y);
            y += Constants.YINC;
            u3.addToStack(u3.isFaceUp(), 2, 2, x, y);
            y += Constants.YINC;
            u4.addToStack(u4.isFaceUp(), 2, 1, x, y);
            y += Constants.YINC;
            u5.addToStack(u5.isFaceUp(), 2, 0, x, y);
            u5.allowDrop(true);

            faceDown.Add(d1);
            faceUp.Add(u1);
            faceUp.Add(u2);
            faceUp.Add(u3);
            faceUp.Add(u4);
            faceUp.Add(u5);
            empty = false;
            totalStack();
        }

        /// <summary>
        /// Column 3
        /// </summary>
        /// <param name="d1">Face-Down Card 1</param>
        /// <param name="d2">Face-Down Card 2</param>
        /// <param name="u1">Face-Up Card 1</param>
        /// <param name="u2">Face-Up Card 2</param>
        /// <param name="u3">Face-Up Card 3</param>
        /// <param name="u4">Face-Up Card 4</param>
        /// <param name="u5">Face-Up Card 5</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card d1, Card d2, Card u1,
            Card u2, Card u3, Card u4, Card u5, 
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 3;
            d1.flip();
            d2.flip();

            d1.addToStack(d1.isFaceUp(), 3, 6, x, y);
            y += Constants.YINCFACEDOWN;
            d2.addToStack(d2.isFaceUp(), 3, 5, x, y);
            y += Constants.YINCFACEDOWN;
            u1.addToStack(u1.isFaceUp(), 3, 4, x, y);
            y += Constants.YINC;
            u2.addToStack(u2.isFaceUp(), 3, 3, x, y);
            y += Constants.YINC;
            u3.addToStack(u3.isFaceUp(), 3, 2, x, y);
            y += Constants.YINC;
            u4.addToStack(u4.isFaceUp(), 3, 1, x, y);
            y += Constants.YINC;
            u5.addToStack(u5.isFaceUp(), 3, 0, x, y);
            u5.allowDrop(true);

            faceDown.Add(d1);
            faceDown.Add(d2);
            faceUp.Add(u1);
            faceUp.Add(u2);
            faceUp.Add(u3);
            faceUp.Add(u4);
            faceUp.Add(u5);
            empty = false;
            totalStack();
        }

        /// <summary>
        /// Column 4
        /// </summary>
        /// <param name="d1">Face-Down Card 1</param>
        /// <param name="d2">Face-Down Card 2</param>
        /// <param name="d3">Face-Down Card 3</param>
        /// <param name="u1">Face-Up Card 1</param>
        /// <param name="u2">Face-Up Card 2</param>
        /// <param name="u3">Face-Up Card 3</param>
        /// <param name="u4">Face-Up Card 4</param>
        /// <param name="u5">Face-Up Card 5</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card d1, Card d2, Card d3, Card u1,
            Card u2, Card u3, Card u4, Card u5,
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 4;
            d1.flip();
            d2.flip();
            d3.flip();

            d1.addToStack(d1.isFaceUp(), 4, 7, x, y);
            y += Constants.YINCFACEDOWN;
            d2.addToStack(d2.isFaceUp(), 4, 6, x, y);
            y += Constants.YINCFACEDOWN;
            d3.addToStack(d3.isFaceUp(), 4, 5, x, y);
            y += Constants.YINCFACEDOWN;
            u1.addToStack(u1.isFaceUp(), 4, 4, x, y);
            y += Constants.YINC;
            u2.addToStack(u2.isFaceUp(), 4, 3, x, y);
            y += Constants.YINC;
            u3.addToStack(u3.isFaceUp(), 4, 2, x, y);
            y += Constants.YINC;
            u4.addToStack(u4.isFaceUp(), 4, 1, x, y);
            y += Constants.YINC;
            u5.addToStack(u5.isFaceUp(), 4, 0, x, y);
            u5.allowDrop(true);

            faceDown.Add(d1);
            faceDown.Add(d2);
            faceDown.Add(d3);
            faceUp.Add(u1);
            faceUp.Add(u2);
            faceUp.Add(u3);
            faceUp.Add(u4);
            faceUp.Add(u5);
            empty = false;
            totalStack();
        }

        /// <summary>
        /// Column 5
        /// </summary>
        /// <param name="d1">Face-Down Card 1</param>
        /// <param name="d2">Face-Down Card 2</param>
        /// <param name="d3">Face-Down Card 3</param>
        /// <param name="d4">Face-Down Card 4</param>
        /// <param name="u1">Face-Up Card 1</param>
        /// <param name="u2">Face-Up Card 2</param>
        /// <param name="u3">Face-Up Card 3</param>
        /// <param name="u4">Face-Up Card 4</param>
        /// <param name="u5">Face-Up Card 5</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card d1, Card d2, Card d3, Card d4,
            Card u1, Card u2, Card u3, Card u4, Card u5,
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 5;
            d1.flip();
            d2.flip();
            d3.flip();
            d4.flip();

            d1.addToStack(d1.isFaceUp(), 5, 8, x, y);
            y += Constants.YINCFACEDOWN;
            d2.addToStack(d2.isFaceUp(), 5, 7, x, y);
            y += Constants.YINCFACEDOWN;
            d3.addToStack(d3.isFaceUp(), 5, 6, x, y);
            y += Constants.YINCFACEDOWN;
            d4.addToStack(d4.isFaceUp(), 5, 5, x, y);
            y += Constants.YINCFACEDOWN;
            u1.addToStack(u1.isFaceUp(), 5, 4, x, y);
            y += Constants.YINC;
            u2.addToStack(u2.isFaceUp(), 5, 3, x, y);
            y += Constants.YINC;
            u3.addToStack(u3.isFaceUp(), 5, 2, x, y);
            y += Constants.YINC;
            u4.addToStack(u4.isFaceUp(), 5, 1, x, y);
            y += Constants.YINC;
            u5.addToStack(u5.isFaceUp(), 5, 0, x, y);
            u5.allowDrop(true);

            faceDown.Add(d1);
            faceDown.Add(d2);
            faceDown.Add(d3);
            faceDown.Add(d4);
            faceUp.Add(u1);
            faceUp.Add(u2);
            faceUp.Add(u3);
            faceUp.Add(u4);
            faceUp.Add(u5);
            empty = false;
            totalStack();
        }

        /// <summary>
        /// Column 6
        /// </summary>
        /// <param name="d1">Face-Down Card 1</param>
        /// <param name="d2">Face-Down Card 2</param>
        /// <param name="d3">Face-Down Card 3</param>
        /// <param name="d4">Face-Down Card 4</param>
        /// <param name="d5">Face-Down Card 5</param>
        /// <param name="u1">Face-Up Card 1</param>
        /// <param name="u2">Face-Up Card 2</param>
        /// <param name="u3">Face-Up Card 3</param>
        /// <param name="u4">Face-Up Card 4</param>
        /// <param name="u5">Face-Up Card 5</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card d1, Card d2, Card d3, Card d4, Card d5,
            Card u1, Card u2, Card u3, Card u4, Card u5,
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 6;
            d1.flip();
            d2.flip();
            d3.flip();
            d4.flip();
            d5.flip();

            d1.addToStack(d1.isFaceUp(), 6, 9, x, y);
            y += Constants.YINCFACEDOWN;
            d2.addToStack(d2.isFaceUp(), 6, 8, x, y);
            y += Constants.YINCFACEDOWN;
            d3.addToStack(d3.isFaceUp(), 6, 7, x, y);
            y += Constants.YINCFACEDOWN;
            d4.addToStack(d4.isFaceUp(), 6, 6, x, y);
            y += Constants.YINCFACEDOWN;
            d5.addToStack(d5.isFaceUp(), 6, 5, x, y);
            y += Constants.YINCFACEDOWN;
            u1.addToStack(u1.isFaceUp(), 6, 4, x, y);
            y += Constants.YINC;
            u2.addToStack(u2.isFaceUp(), 6, 3, x, y);
            y += Constants.YINC;
            u3.addToStack(u3.isFaceUp(), 6, 2, x, y);
            y += Constants.YINC;
            u4.addToStack(u4.isFaceUp(), 6, 1, x, y);
            y += Constants.YINC;
            u5.addToStack(u5.isFaceUp(), 6, 0, x, y);
            u5.allowDrop(true);

            faceDown.Add(d1);
            faceDown.Add(d2);
            faceDown.Add(d3);
            faceDown.Add(d4);
            faceDown.Add(d5);
            faceUp.Add(u1);
            faceUp.Add(u2);
            faceUp.Add(u3);
            faceUp.Add(u4);
            faceUp.Add(u5);
            empty = false;
            totalStack();
        }

        /// <summary>
        /// Column 7
        /// </summary>
        /// <param name="d1">Face-Down Card 1</param>
        /// <param name="d2">Face-Down Card 2</param>
        /// <param name="d3">Face-Down Card 3</param>
        /// <param name="d4">Face-Down Card 4</param>
        /// <param name="d5">Face-Down Card 5</param>
        /// <param name="d6">Face-Down Card 6</param>
        /// <param name="u1">Face-Up Card 1</param>
        /// <param name="u2">Face-Up Card 2</param>
        /// <param name="u3">Face-Up Card 3</param>
        /// <param name="u4">Face-Up Card 4</param>
        /// <param name="u5">Face-Up Card 5</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Column(Card d1, Card d2, Card d3, Card d4, Card d5, Card d6,
            Card u1, Card u2, Card u3, Card u4, Card u5,
            int x, int y)
        {
            faceDown = new ArrayList();
            faceUp = new ArrayList();
            stack = new ArrayList();
            whichCol = 7;
            d1.flip();
            d2.flip();
            d3.flip();
            d4.flip();
            d5.flip();
            d6.flip();

            d1.addToStack(d1.isFaceUp(), 7, 10, x, y);
            y += Constants.YINCFACEDOWN;
            d2.addToStack(d2.isFaceUp(), 7, 9, x, y);
            y += Constants.YINCFACEDOWN;
            d3.addToStack(d3.isFaceUp(), 7, 8, x, y);
            y += Constants.YINCFACEDOWN;
            d4.addToStack(d4.isFaceUp(), 7, 7, x, y);
            y += Constants.YINCFACEDOWN;
            d5.addToStack(d5.isFaceUp(), 7, 6, x, y);
            y += Constants.YINCFACEDOWN;
            d6.addToStack(d6.isFaceUp(), 7, 5, x, y);
            y += Constants.YINCFACEDOWN;
            u1.addToStack(u1.isFaceUp(), 7, 4, x, y);
            y += Constants.YINC;
            u2.addToStack(u2.isFaceUp(), 7, 3, x, y);
            y += Constants.YINC;
            u3.addToStack(u3.isFaceUp(), 7, 2, x, y);
            y += Constants.YINC;
            u4.addToStack(u4.isFaceUp(), 7, 1, x, y);
            y += Constants.YINC;
            u5.addToStack(u5.isFaceUp(), 7, 0, x, y);
            u5.allowDrop(true);

            faceDown.Add(d1);
            faceDown.Add(d2);
            faceDown.Add(d3);
            faceDown.Add(d4);
            faceDown.Add(d5);
            faceDown.Add(d6);
            faceUp.Add(u1);
            faceUp.Add(u2);
            faceUp.Add(u3);
            faceUp.Add(u4);
            faceUp.Add(u5);
            empty = false;
            totalStack();
        }
        #endregion

        public void addStack(ArrayList newStack)
        {
            Card c;
            for (int i = 0; i < newStack.Count; i++)
            {
                c = (Card)newStack[i];
                faceUp.Add(c);
            }
            checkState();
        }

        public void removeStack(ArrayList oldStack)
        {
            for (int j = 0; j < oldStack.Count; j++)
            {
                for (int i = 0; i < faceUp.Count; i++)
                {
                    if (((Card)faceUp[i]).toString().Equals(
                        ((Card)oldStack[j]).toString() ) )
                    {
                        faceUp.RemoveAt(i);
                    }
                }
            }
            checkState();
        }

        public Card getCard(int num)
        {
            return ((Card)stack[num]);
        }

        public ArrayList getColumn()
        {
            return stack;
        }

        public ArrayList getColumn(int numCards)
        {
            ArrayList temp = new ArrayList();
            for (int x = stack.Count - numCards - 1; x < stack.Count; x++)
            {
                temp.Add((Card)stack[x]);
            }

            return temp;
        }

        public void flip()
        {
            Card cardToFlip = (Card)faceDown[faceDown.Count - 1];
            cardToFlip.flip();
            faceDown.Remove(cardToFlip);
            faceUp.Add(cardToFlip);
            checkState();
        }

        private void checkState()
        {

            checkEmptiness();
            totalStack();
        }

        public bool needPlaceholder()
        {
            if (faceUp.Count == 0)
            {
                if (faceDown.Count == 0)
                {
                    empty = true;
                    if (whichCol > 1) {
                        Card c = new Card(whichCol, Constants.COL3X, Constants.YBEGIN, false, "place");
                    }
                }
                else
                    empty = false;
            }
            else
            {
                empty = false;
            }
            return empty;
        }

        private void checkEmptiness()
        {
            if (faceUp.Count == 0)
            {
                if (faceDown.Count == 0)
                {
                    empty = true;
                  //  Card c = new Card(whichCol, Constants.COL1X, Constants.YBEGIN);
                }
                else
                    empty = false;
            }
            else
            {
                empty = false;
            }
        }

        private bool canAllowKings()
        {
            checkEmptiness();
            return empty;
        }

        private void totalStack()
        {
            stack.Clear();
            for (int i = 0; i < faceDown.Count; i++)
            {
                stack.Add(faceDown[i]);
            }
            for (int j = 0; j < faceUp.Count; j++)
            {
                stack.Add(faceUp[j]);
            }
            int d = stack.Count - 1;
            for (int i = 0; i < stack.Count; i++)
            {
                ((Card)stack[i]).changeStack(whichCol);
                ((Card)stack[i]).allowDrop(false);
                ((Card)stack[i]).changeDepth(d);
                d--;
            }
            if (!canAllowKings() && ((Card)stack[stack.Count - 1]).isFaceUp())
            {
                ((Card)stack[stack.Count - 1]).allowDrop(true);
            }
        }

        private int getNewInc()
        {
            int inc = Constants.YINC;
            int cardsOver = faceUp.Count - 14;
            for (int i = 0; i < cardsOver; i++)
            {
                if (i < 4)
                    inc -= 2;
                else
                    inc -= 1;
            }
            return inc;
        }

        public void scrunchCol(bool doIt)
        {
            if (doIt)
            {
                //Scrunch column
                int newInc = getNewInc();
                int y = Constants.YBEGIN + newInc;
                for (int j = 0; j < faceDown.Count; j++)
                {
                    y += Constants.YINCFACEDOWN;
                } 
                for (int i = 1; i < faceUp.Count; i++)
                {
                    ((Card)faceUp[i]).setLocation(((Card)faceUp[i]).Location.X ,y);
                    y += newInc;
                }
            }
            else
            {
                //Undo Scrunch
                int y = Constants.YBEGIN + Constants.YINC;
                for (int j = 0; j < faceDown.Count; j++)
                {
                    y += Constants.YINCFACEDOWN;
                } 
                for (int i = 1; i < faceUp.Count; i++)
                {
                    ((Card)faceUp[i]).setLocation(((Card)faceUp[i]).Location.X, y);
                    y += Constants.YINC;
                }
            }
        }

        public int colSize()
        {
            return stack.Count;
        }

        public int colUpSize()
        {
            return faceUp.Count;
        }
    }
}
