//------------------------------------------------------------------------------
//  Russian Solitaire! v.1.0
//      2006   Steph Stubler
//              University of Virginia
//              CS 216 - Programming & Data Representation [Prof. David Evans]
//
//              Deck.cs
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Drawing;
using System.Web;

namespace RussianSolitaire
{
    class Deck
    {
        public ArrayList theDeck;
        public Bitmap cardBacks;

        public String time;

        public Deck(Bitmap cardDesign)
        {
            theDeck = new ArrayList();
            cardBacks = cardDesign;

            // INITIALIZE EVERY CARD HERE.
            String[] suit = new String[4];
            suit[0] = Constants.HEARTS;
            suit[1] = Constants.DIAMONDS;
            suit[2] = Constants.CLUBS;
            suit[3] = Constants.SPADES;

            for (int j = 0; j <= 3; j++)
            {
                String currSuit = suit[j];
                for (int i = 1; i <= 13; i++)
                {
                    Card newCard = new Card(i, currSuit, cardBacks);
                    theDeck.Add(newCard);
                }
            }
            
        }

        public Card getCardFromDeck(int n)
        {
            return ((Card) theDeck[n]);
        }

        /// <summary>
        /// If I am ever ambitious, I will allow for users to select different card backs.
        /// </summary>
        /// <param name="cardDesign"></param>
        public void changeDeck(Bitmap cardDesign)
        {
            cardBacks = cardDesign;
            for (int i = 0; i < 52; i++)
            {
                ((Card)theDeck[i]).changeDeck(cardBacks);
            }
        }

        /// <summary>
        /// Method called to shuffle all cards in deck before dealing them to the table.
        ///     In terms of a physical deck, this method will take a card at a random place in the deck,
        ///     then switch this card with another card at a random place in the deck. This occurs a random
        ///     number of times to ensure proper shuffling.
        /// </summary>
        public void shuffle()
        {
            int startTicks = Environment.TickCount;
            int randNum1, randNum2;
            Card temp;
            int until = RandomNumber();
            int seed = until;
            for (int i = 0; i < until; i++)
            {
                randNum1 = RandomNumber(++seed);
                randNum2 = RandomNumber(randNum1);
                temp = ((Card)theDeck[randNum1]);
                theDeck[randNum1] = ((Card)theDeck[randNum2]);
                theDeck[randNum2] = temp;
            }
            int endTicks2 = Environment.TickCount - startTicks;
            time = endTicks2.ToString() + " for 1 shuffles.";
        }

        /// <summary>
        /// Called to determine how many cards will be switched.
        /// </summary>
        /// <returns>Random number between 0 and 1000.</returns>
        private int RandomNumber()
        {
            Random random = new Random(Environment.TickCount);
            return random.Next(0, 1000);
        }
        /// <summary>
        /// This returns a place in the deck.
        /// </summary>
        /// <param name="seed">Passed in random integer, used to obtain different random numbers.</param>
        /// <returns>Random number (0-51).</returns>
        private int RandomNumber(int seed)
        {
            Random random = new Random(seed);
            return random.Next(0, 52);
        }
    }
}
