//------------------------------------------------------------------------------
//  Russian Solitaire! v.1.0
//      2006   Steph Stubler
//              University of Virginia
//              CS 216 - Programming & Data Representation [Prof. David Evans]
//
//              Foundation.cs
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace RussianSolitaire
{
    class Foundation
    {
        public ArrayList hearts, diamonds, spades, clubs;
        public int score;

        public Foundation()
        {
            hearts = new ArrayList();
            diamonds = new ArrayList();
            spades = new ArrayList();
            clubs = new ArrayList();
            score = 0;
        }

        /// <summary>
        /// Checks to see if specified card is "ready" to go to foundation.
        /// </summary>
        /// <param name="newCard">Card trying to move.</param>
        /// <param name="doIt">If true, move it. If false, just check if possible.</param>
        /// <param name="debug">If true, card values don't matter.</param>
        /// <returns>True if card can be placed in foundation.</returns>
        public bool isReady(Card newCard, bool doIt, bool debug)
        {
            bool ret = false;
            if (!debug)
            {
                if (newCard.getSuit().Equals(Constants.HEARTS))
                {
                    int currentVal;
                    if (getHeartsCount() == 0)
                        currentVal = 0;
                    else
                        currentVal = ((Card)hearts[hearts.Count - 1]).getValue();
                    int newValue = newCard.getValue();
                    if ((newValue - currentVal) == 1)
                    {
                        if (doIt)
                        {
                            hearts.Add(newCard);
                            score += 5;
                        }
                        ret = true;
                    }
                }
                else if (newCard.getSuit().Equals(Constants.DIAMONDS))
                {
                    int currentVal;
                    if (getDiamondsCount() == 0)
                        currentVal = 0;
                    else
                        currentVal = ((Card)diamonds[diamonds.Count - 1]).getValue();
                    int newValue = newCard.getValue();
                    if ((newValue - currentVal) == 1)
                    {
                        if (doIt)
                        {
                            diamonds.Add(newCard);
                            score += 5;
                        }
                        ret = true;
                    }
                }
                else if (newCard.getSuit().Equals(Constants.CLUBS))
                {
                    int currentVal;
                    if (getClubsCount() == 0)
                        currentVal = 0;
                    else
                        currentVal = ((Card)clubs[clubs.Count - 1]).getValue();
                    int newValue = newCard.getValue();
                    if ((newValue - currentVal) == 1)
                    {
                        if (doIt)
                        {
                            clubs.Add(newCard);
                            score += 5;
                        }
                        ret = true;
                    }
                }
                else
                {
                    int currentVal;
                    if (getSpadesCount() == 0)
                        currentVal = 0;
                    else
                        currentVal = ((Card)spades[spades.Count - 1]).getValue();
                    int newValue = newCard.getValue();
                    if ((newValue - currentVal) == 1)
                    {
                        if (doIt)
                        {
                            spades.Add(newCard);
                            score += 5;
                        }
                        ret = true;
                    }
                }
            }
            else // Debug is on, let any one card drop onto foundation.
            {
                if (newCard.getSuit().Equals(Constants.HEARTS))
                {
                    if (doIt)
                        hearts.Add(newCard);
                    ret = true;
                }
                else if (newCard.getSuit().Equals(Constants.DIAMONDS))
                {
                    if (doIt)
                        diamonds.Add(newCard);
                    ret = true;
                }
                else if (newCard.getSuit().Equals(Constants.CLUBS))
                {
                    if (doIt)
                        clubs.Add(newCard);
                    ret = true;
                }
                else
                {
                    if (doIt)
                        spades.Add(newCard);
                    ret = true;
                }
            }
            if (doIt)
               setDepths();
            return ret;
        }

        public void setDepths()
        {
            int depth = hearts.Count - 1;
            for (int i = 0; i < hearts.Count; i++)
            {
                ((Card)hearts[i]).changeDepth(depth);
                depth--;
            }
            depth = diamonds.Count - 1;
            for (int i = 0; i < diamonds.Count; i++)
            {
                ((Card)diamonds[i]).changeDepth(depth);
                depth--;
            }
            depth = clubs.Count - 1;
            for (int i = 0; i < clubs.Count; i++)
            {
                ((Card)clubs[i]).changeDepth(depth);
                depth--;
            }
            depth = spades.Count - 1;
            for (int i = 0; i < spades.Count; i++)
            {
                ((Card)spades[i]).changeDepth(depth);
                depth--;
            }
        }

        #region Hearts
        /// <summary>
        /// Returns second-most top card (because new card has already been added.
        /// </summary>
        /// <returns>Second most top card.</returns>
        public Card getHeartsTop()
        {
            return ((Card) hearts[hearts.Count - 2]);
        }

        public int getHeartsCount()
        {
            return hearts.Count;
        }

        public bool checkHearts()
        {
            if (hearts.Count == 13)
                return true;
            else
                return false;
        }
        #endregion

        #region Diamonds
        /// <summary>
        /// Returns second-most top card (because new card has already been added.
        /// </summary>
        /// <returns>Second-most top card.</returns>
        public Card getDiamondsTop()
        {
            return ((Card)diamonds[diamonds.Count - 2]);
        }

        public int getDiamondsCount()
        {
            return diamonds.Count;
        }

        public bool checkDiamonds()
        {
            if (diamonds.Count == 13)
                return true;
            else
                return false;
        }
        #endregion

        #region Clubs
        /// <summary>
        /// Returns second-most top card (because new card has already been added.
        /// </summary>
        /// <returns>Second-most top card.</returns>
        public Card getClubsTop()
        {
            return ((Card)clubs[clubs.Count - 2]);
        }

        public int getClubsCount()
        {
            return clubs.Count;
        }

        public bool checkClubs()
        {
            if (clubs.Count == 13)
                return true;
            else
                return false;
        }
        #endregion

        #region Spades
        /// <summary>
        /// Returns second-most top card (because new card has already been added.
        /// </summary>
        /// <returns>Second-most top card.</returns>
        public Card getSpadesTop()
        {
            return ((Card)spades[spades.Count - 2]);
        }

        public int getSpadesCount()
        {
            return spades.Count;
        }

        public bool checkSpades()
        {
            if (spades.Count == 13)
                return true;
            else
                return false;
        }
        #endregion
    }
}
