//------------------------------------------------------------------------------
//  Russian Solitaire! v.1.0
//      2006   Steph Stubler
//              University of Virginia
//              CS 216 - Programming & Data Representation [Prof. David Evans]
//
//              GUI.cs
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Resources;
using DragDrop;


namespace RussianSolitaire
{
    public partial class GUI : Form
    {
        public bool debug = false;

        Table t;
        // Cards that are playable.
        Bitmap backs;
        ArrayList tableau;
        // Cards that should be cleared to play again.
        ArrayList cardsToClear;
        // Number of possible moves at any given moment.
        int more;
        bool mouseDowned;
        bool mouseMoved;
        bool peeking;

        private MouseMove mMove;

        public GUI()
        {
            backs = global::RussianSolitaire.Properties.Resources.aCardBackRed;
            t = new Table(backs);
            tableau = new ArrayList();
            cardsToClear = new ArrayList();
            mouseDowned = false;
            mouseMoved = false;

            InitializeComponent();

            refreshCols();
        }

        private void refreshCols()
        {
            buildCols();
            mMove = new MouseMove(clock);
            checkRemainingMoves.Enabled = true;
        }

        /// <summary>
        /// Makes 4 placeholders for foundation, 7 placeholder labels for tableau,
        ///     builds all 7 tableau piles.
        /// </summary>
        private void buildCols()
        {
            buildCol1(t.col1);
            buildCol2(t.col2);
            buildCol3(t.col3);
            buildCol4(t.col4);
            buildCol5(t.col5);
            buildCol6(t.col6);
            buildCol7(t.col7);

            makePlaceholder(t.f1, true);
            makePlaceholder(t.f2, true);
            makePlaceholder(t.f3, true);
            makePlaceholder(t.f4, true);
            this.Controls.Add(t.f1);
            this.Controls.Add(t.f2);
            this.Controls.Add(t.f3);
            this.Controls.Add(t.f4);

            makePlaceholderLabel(Constants.COL1X, Constants.YBEGIN, "place");
            makePlaceholderLabel(Constants.COL2X, Constants.YBEGIN, "place");
            makePlaceholderLabel(Constants.COL3X, Constants.YBEGIN, "place");
            makePlaceholderLabel(Constants.COL4X, Constants.YBEGIN, "place");
            makePlaceholderLabel(Constants.COL5X, Constants.YBEGIN, "place");
            makePlaceholderLabel(Constants.COL6X, Constants.YBEGIN, "place");
            makePlaceholderLabel(Constants.COL7X, Constants.YBEGIN, "place");
        }

        #region Program Menu Items

        private void undoLastMoveToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }
        
        private void newGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            newGame();
        }
        
        public void newGame()
        {
            int startTicks = Environment.TickCount;
            foreach (Control i in cardsToClear)
            {
                this.Controls.Remove(i);
            }
            int endTicks2 = Environment.TickCount - startTicks;
            startTicks = Environment.TickCount;
            t = new Table(backs);
            endTicks2 = Environment.TickCount - startTicks;
            tableau.Clear();
            refreshCols();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void highScoresToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //HighScoreTable hs = new HighScoreTable(this);
           // hs.Show();
        }

        private void rulesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RulesBox rb = new RulesBox();
            rb.Show();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox ab = new AboutBox();
            ab.Show();
        }

        private void debugToolStripMenuItem_Click(object sender, EventArgs e)
        {
            debug = !debug;
            /*
            if (debug)
                debugOnToolStripMenuItem.Text = "&Debug Off";
            else
                debugOnToolStripMenuItem.Text = "&Debug On";
             * */
        }

        #endregion

        #region Building Columns
        private void buildCol1(Column col)
        {
            cardsToClear.Add(t.p1);
            this.Controls.Add(t.p1);
            if (t.p1.placeHolderActive())
                tableau.Add(t.p1);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        private void buildCol2(Column col)
        {
            cardsToClear.Add(t.p2);
            this.Controls.Add(t.p2);
            if (t.p2.placeHolderActive())
                tableau.Add(t.p2);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        private void buildCol3(Column col)
        {
            cardsToClear.Add(t.p3);
            this.Controls.Add(t.p3);
            if (t.p3.placeHolderActive())
                tableau.Add(t.p3);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        private void buildCol4(Column col)
        {
            cardsToClear.Add(t.p4);
            this.Controls.Add(t.p4);
            if (t.p4.placeHolderActive())
                tableau.Add(t.p4);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        private void buildCol5(Column col)
        {
            cardsToClear.Add(t.p5);
            this.Controls.Add(t.p5);
            if (t.p5.placeHolderActive())
                tableau.Add(t.p5);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        private void buildCol6(Column col)
        {
            cardsToClear.Add(t.p6);
            this.Controls.Add(t.p6);
            if (t.p6.placeHolderActive())
                tableau.Add(t.p6);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        private void buildCol7(Column col)
        {
            cardsToClear.Add(t.p7);
            this.Controls.Add(t.p7);
            if (t.p7.placeHolderActive())
                tableau.Add(t.p7);
            for (int j = col.getColumn().Count - 1; j >= 0; j--)
            {
                Card d = (Card)col.getCard(j);
                d.MouseDown += new MouseEventHandler(dragme_MouseDown);
                d.MouseUp += new MouseEventHandler(dragme_MouseUp);
                d.MouseMove += new MouseEventHandler(dragme_MouseMove);
                d.Click += new EventHandler(facedown_Click);
                d.DoubleClick += new EventHandler(tofoundation_DoubleClick);
                d.MouseEnter += new EventHandler(dragme_MouseEnter);
                d.MouseLeave += new EventHandler(dragme_MouseLeave);

                this.Controls.Add(d);
                tableau.Add(d);
                cardsToClear.Add(d);
            }
        }
        #endregion

        #region Mouse Event Handlers
        private void dragme_MouseDown(object sender, MouseEventArgs e)
        {
            Control c = (Control)sender;
            if (((Card)sender).canMove())
            {
                if (peeking)
                {
                    peeking = false;
                    mMove.CardPeekDown((Control)sender);
                }
                ArrayList cardStack = new ArrayList();
                //debug = true;
                cardStack = dragStack(c);
                if (e.Button == MouseButtons.Left)
                {
                    mMove.Begin(cardStack, e.X, e.Y, ((Card)sender).Size.Height + 2);
                }
                mouseDowned = true;
            }
        }

        private void dragme_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (mMove.isMoving() && (mouseMoved))
            {
                #region Mouse has been down and has moved.
                if ((e.Button == MouseButtons.Left))
                {
                    if (((Card)sender).canMove())
                    {
                        mMove.EndIt();
                        mMove.setEnd();
                        int x = mMove.getEndX();
                        int y = mMove.getEndY();
                        //debug = true;
                        if (!CheckDrop(x, y, sender, debug))
                        {
                            if (!((Card)sender).isFoundation()) //Makes sure foundation cards don't move.
                            {
                                mMove.ToOldPosition();
                            }
                        }
                       // debug = false;
                    }
                }
                #endregion
            }
            mouseDowned = false;
            mouseMoved = false;
            checkForScrunch();
        }

        private void dragme_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                if (((Card)sender).canMove())
                    mMove.InProgress(e.X, e.Y);
                if (mouseDowned)
                {
                    mouseMoved = true;
                }
            }
        }

        private void dragme_MouseEnter(object sender, EventArgs e)
        {
            if ((((Card)sender).canMove()) && (!mMove.isMoving()))
            {
                // If card is on tableau and mouse runs over it, it raises a little.
                peeking = true;
                mMove.CardPeekUp((Control)sender, 7);
            }
        }

        private void dragme_MouseLeave(object sender, EventArgs e)
        {
            if ((((Card)sender).canMove()) && (!mMove.isMoving()))
            {
                if (peeking)
                {
                    // If card is on tableau and mouse runs over it, it raises a little.
                    mMove.CardPeekDown((Control)sender);
                    peeking = false;
                }
            }
        }

        private void facedown_Click(object sender, EventArgs e)
        {
            if (!(((Card)sender).isFaceUp()))
            {
                int whichStack = ((Card)sender).getStack();
                int depth = ((Card)sender).getDepth();

                #region Make sure card is on bottom of stack
                if (depth == 0) // Card is on bottom, ready to flip over.
                {
                    if (whichStack == 2)
                    {
                        t.col2.flip();
                    }
                    else if (whichStack == 3)
                    {
                        t.col3.flip();
                    }
                    else if (whichStack == 4)
                    {
                        t.col4.flip();
                    }
                    else if (whichStack == 5)
                    {
                        t.col5.flip();
                    }
                    else if (whichStack == 6)
                    {
                        t.col6.flip();
                    }
                    else if (whichStack == 7)
                    {
                        t.col7.flip();
                    }
                }
                #endregion
            }
        }

        private void tofoundation_DoubleClick(object sender, EventArgs e)
        {
            int depth = ((Card)sender).getDepth();
            bool ret;
            if (peeking)
            {
                peeking = false;
                mMove.CardPeekDown((Control)sender);
            }
            if (((Card)sender).isFaceUp() && (!((Card)sender).isFoundation()) &&
                (!((Card)sender).isPlaceholder()))
            {
                if (depth == 0)
                {
                    #region Card is on the Foundation
                    ret = t.found.isReady((Card)sender, true, false);
                    if (ret)
                    #region If Card was Dropped onto Foundation (no Debug)
                    {
                        int s, count;
                        Card oldCard;
                        if (((Card)sender).getSuit().Equals(
                            Constants.HEARTS))
                        {
                            s = Constants.FOUND1X;
                            count = t.found.getHeartsCount();
                            if (count > 1)
                                oldCard = (Card)t.found.getHeartsTop();
                            else
                                oldCard = (Card)t.f1;
                        }
                        else if (((Card)sender).getSuit().Equals(
                            Constants.CLUBS))
                        {
                            s = Constants.FOUND2X;
                            count = t.found.getClubsCount();
                            if (count > 1)
                                oldCard = (Card)t.found.getClubsTop();
                            else
                                oldCard = (Card)t.f2;
                        }
                        else if (((Card)sender).getSuit().Equals(
                            Constants.DIAMONDS))
                        {
                            s = Constants.FOUND3X;
                            count = t.found.getDiamondsCount();
                            if (count > 1)
                                oldCard = (Card)t.found.getDiamondsTop();
                            else
                                oldCard = (Card)t.f3;
                        }
                        else
                        {
                            s = Constants.FOUND4X;
                            count = t.found.getSpadesCount();
                            if (count > 1)
                                oldCard = (Card)t.found.getSpadesTop();
                            else
                                oldCard = (Card)t.f4;
                        }
                        Card copyCard = (Card) oldCard;
                        makeFoundation((Card)sender);
                        makePlaceholder(oldCard, false);
                        mMove.SnapToNewPosition(s,
                            (copyCard.Location.Y - Constants.YINC),
                            Constants.YINC);
                        ArrayList temp = new ArrayList();
                        temp.Add((Card)sender);
                        updateColumn(temp, 8,
                            (int)((Card)sender).getStack());
                        mMove.EndIt();
                    }
                    #endregion
                    #endregion
                }
            }
        }

        private void clock_Tick(object sender, EventArgs e)
        {
            mMove.ToNewPosition();
        }

        private void checkRemainingMoves_Tick(object sender, EventArgs e)
        {
            checkForMoves();
        }
        #endregion

        /// <summary>
        /// Builds an Arraylist with all cards being dragged.
        /// </summary>
        /// <param name="c">First card in stack.</param>
        /// <param name="debug">If true, allows only first card to be dragged.</param>
        /// <returns>Arraylist with all cards being dragged.</returns>
        private ArrayList dragStack(Control c)
        {
            ArrayList stack = new ArrayList();
            int whichStack = ((Card)c).getStack();
            int depth = ((Card)c).getDepth();

            ArrayList temp = new ArrayList();
            if (whichStack == 1)
            {
                stack = t.col1.getColumn(depth);
            }
            else if (whichStack == 2)
            {
                stack = t.col2.getColumn(depth);
            }
            else if (whichStack == 3)
            {
                stack = t.col3.getColumn(depth);
            }
            else if (whichStack == 4)
            {
                stack = t.col4.getColumn(depth);
            }
            else if (whichStack == 5)
            {
                stack = t.col5.getColumn(depth);
            }
            else if (whichStack == 6)
            {
                stack = t.col6.getColumn(depth);
            }
            else if (whichStack == 7)
            {
                stack = t.col7.getColumn(depth);
            }
            else
            {
                stack = t.col1.getColumn(depth);
            }
            return stack;
        }

        /// <summary>
        /// Verifies whether or not user drop is legal. If yes, card stack being
        ///     dragged snaps underneath new stack. If no, card stack snaps back
        ///     to its original position.
        /// </summary>
        /// <param name="x">Where mouse is.</param>
        /// <param name="y">Where mouse is.</param>
        /// <param name="send">First card of dragged stack.</param>
        /// <param name="debug">If on, allows for dragged stack to move to any
        ///                         new stack.</param>
        /// <returns>Whether or not user drop is legal.</returns>
        private bool CheckDrop(int x, int y, object send, bool debug)
        {
            bool ret = false;
            Control c;
            Control sender = (Control) send;

            foreach (Control i in tableau)
            {
                if (! (((Card)i).toString().Equals(
                    ((Card)sender).toString())) )
                {
                    int x2, y2, sizeX, sizeY;
                    x2 = i.Location.X;
                    y2 = i.Location.Y;
                    sizeX = i.Size.Width;
                    sizeY = i.Size.Height;
                    if ((x >= (x2 - (Constants.CARDWIDTH / 2)) && x <= ((x2 + sizeX) - (Constants.CARDWIDTH / 2)) &&
                        (y >= y2 && y <= (y2 + sizeY))))
                    {
                        if (i.AllowDrop)
                        {
                            c = i;
                            //check if the move is valid.
                            if (!debug)
                            {
                                if (((Card)c).isPlaceholder())
                                {
                                    if (((Card)c).isFoundation())
                                    {
                                        #region Card is on the Foundation
                                        ArrayList cardStack = dragStack(sender);
                                        if (cardStack.Count == 1)
                                        {
                                            ret = t.found.isReady((Card)cardStack[0], true, false);
                                            if (ret)
                                            #region If Card was Dropped onto Foundation (no Debug)
                                            {
                                                int s, count;
                                                Card oldCard;
                                                if (((Card)sender).getSuit().Equals(
                                                    Constants.HEARTS))
                                                {
                                                    s = Constants.FOUND1X;
                                                    count = t.found.getHeartsCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getHeartsTop();
                                                    else
                                                        oldCard = (Card)t.f1;
                                                }
                                                else if (((Card)sender).getSuit().Equals(
                                                    Constants.CLUBS))
                                                {
                                                    s = Constants.FOUND2X;
                                                    count = t.found.getClubsCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getClubsTop();
                                                    else
                                                        oldCard = (Card)t.f2;
                                                }
                                                else if (((Card)sender).getSuit().Equals(
                                                    Constants.DIAMONDS))
                                                {
                                                    s = Constants.FOUND3X;
                                                    count = t.found.getDiamondsCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getDiamondsTop();
                                                    else
                                                        oldCard = (Card)t.f3;
                                                }
                                                else
                                                {
                                                    s = Constants.FOUND4X;
                                                    count = t.found.getSpadesCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getSpadesTop();
                                                    else
                                                        oldCard = (Card)t.f4;
                                                }
                                                makeFoundation((Card)sender);
                                                makePlaceholder(oldCard, false);
                                                mMove.SnapToNewPosition(s,
                                                    (((Card)c).Location.Y - Constants.YINC),
                                                    Constants.YINC);
                                                updateColumn(cardStack, 8,
                                                    (int)((Card)sender).getStack());
                                            }
                                            #endregion
                                            break;
                                        }
                                        #endregion
                                    }
                                    else
                                    {
                                        #region New Card is not on the Foundation
                                        if (((Card)sender).getValue() == 13)
                                        {
                                            mMove.SnapToNewPosition(((Card)c).Location.X,
                                                (((Card)c).Location.Y - Constants.YINC),
                                                Constants.YINC);
                                            ArrayList cardStack = dragStack(sender);
                                            updateColumn(cardStack,
                                                (int)((Card)c).getStack(),
                                                (int)((Card)sender).getStack());
                                            ((Card)c).setPlaceholderActive(false);
                                            ret = true;
                                            break;
                                        }
                                        #endregion
                                    }
                                }
                                else
                                {
                                    #region Card is not a Placeholder
                                    if ((((Card)c).getSuit().Equals(((Card)sender).getSuit()) &&
                                        ((((Card)c).getValue() - ((Card)sender).getValue() == 1))))
                                    {
                                        mMove.SnapToNewPosition(((Card)c).Location.X, ((Card)c).Location.Y,
                                            Constants.YINC);
                                        ArrayList cardStack = dragStack(sender);
                                        updateColumn(cardStack,
                                            (int)((Card)c).getStack(),
                                            (int)((Card)sender).getStack());
                                        ret = true;
                                        break;
                                    }
                                    #endregion
                                }
                            }
                            else // DEBUG is ON, anything goes, regardless of value.
                            {
                                if (((Card)c).isPlaceholder())
                                {
                                    if (((Card)c).isFoundation())
                                    {
                                        #region New Card is on the Foundation
                                        ArrayList cardStack = dragStack(sender);
                                        if (cardStack.Count == 1)
                                        {
                                            ret = t.found.isReady((Card)cardStack[0], true, true);
                                            if (ret)
                                            #region If Card was Dropped onto Foundation (with Debug)
                                            {
                                                int s, count;
                                                Card oldCard;
                                                if (((Card)sender).getSuit().Equals(
                                                    Constants.HEARTS))
                                                {
                                                    s = Constants.FOUND1X;
                                                    count = t.found.getHeartsCount();
                                                    if (count > 1)
                                                        oldCard = (Card) t.found.getHeartsTop();
                                                    else
                                                        oldCard = (Card) t.f1;
                                                }
                                                else if (((Card)sender).getSuit().Equals(
                                                    Constants.CLUBS))
                                                {
                                                    s = Constants.FOUND2X;
                                                    count = t.found.getClubsCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getClubsTop();
                                                    else
                                                        oldCard = (Card)t.f2;
                                                }
                                                else if (((Card)sender).getSuit().Equals(
                                                    Constants.DIAMONDS))
                                                {
                                                    s = Constants.FOUND3X;
                                                    count = t.found.getDiamondsCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getDiamondsTop();
                                                    else
                                                        oldCard = (Card)t.f3;
                                                }
                                                else
                                                {
                                                    s = Constants.FOUND4X;
                                                    count = t.found.getSpadesCount();
                                                    if (count > 1)
                                                        oldCard = (Card)t.found.getSpadesTop();
                                                    else
                                                        oldCard = (Card)t.f4;
                                                }
                                                makeFoundation((Card) sender);
                                                makePlaceholder(oldCard, false);
                                                mMove.SnapToNewPosition(s,
                                                    (((Card)c).Location.Y - Constants.YINC),
                                                    Constants.YINC);
                                                updateColumn(cardStack, 8,
                                                    (int)((Card)sender).getStack());
                                            }
                                            #endregion
                                            break;
                                        #endregion
                                        }
                                    }
                                    else
                                    {
                                        #region New Card is not on the Foundation
                                        mMove.SnapToNewPosition(((Card)c).Location.X,
                                            (((Card)c).Location.Y - Constants.YINC),
                                            Constants.YINC);
                                        ArrayList cardStack = dragStack(sender);
                                        updateColumn(cardStack,
                                            (int)((Card)c).getStack(),
                                            (int)((Card)sender).getStack());
                                        ((Card)c).setPlaceholderActive(false);
                                        ret = true;
                                        break;
                                        #endregion
                                    }
                                }
                                else
                                {
                                    #region Card is not a PlaceHolder
                                    mMove.SnapToNewPosition(((Card)c).Location.X, ((Card)c).Location.Y,
                                            Constants.YINC);
                                    ArrayList cardStack = dragStack(sender);
                                    updateColumn(cardStack,
                                        (int)((Card)c).getStack(),
                                        (int)((Card)sender).getStack());
                                    ret = true;
                                    break;
                                    #endregion
                                }
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Used when card successfully enters foundation.
        /// </summary>
        /// <param name="c">Card being moved into foundation</param>
        private void makeFoundation(Card c)
        {
            c.setFoundation();
        }

        /// <summary>
        /// Used when all cards have left a tableau pile, a placeholder holds a place so that
        ///        Kings can be dropped onto them.
        /// </summary>
        /// <param name="c">Placeholder</param>
        /// <param name="doIt">True if you want to do it, false if changing card back into a mere label.</param>
        private void makePlaceholder(Card c, bool doIt)
        {
            c.setPlaceholderActive(doIt);
            if (doIt)
                tableau.Add(c);
            else
            {
                tableau.Remove(c);
                c.Visible = false;
                this.Controls.Remove(c);
                cardsToClear.Remove(c);
            }
        }

        /// <summary>
        /// Appearance of placeholder, has no other useful functionality.
        /// </summary>
        /// <param name="x">X position on GUI.</param>
        /// <param name="y">Y position on GUI.</param>
        /// <param name="image"></param>
        private void makePlaceholderLabel(int x, int y, String image)
        {
            Label l = new Label();
            l.Location = new Point(x, y);
            ResourceManager rm = new ResourceManager("RussianSolitaire.Properties.Resources", GetType().Assembly);
            Bitmap b;
            b = (Bitmap)rm.GetObject(image);

            Size cardSize = new Size(Constants.CARDWIDTH, Constants.CARDHEIGHT);
            l.Size = cardSize;
            l.AllowDrop = false;
            //l.Text = "-";
            l.TextAlign = ContentAlignment.MiddleCenter;
            l.Image = b;
            l.BorderStyle = BorderStyle.None;
            l.ForeColor = Color.Black;
            l.SendToBack();
            this.Controls.Add(l);
            cardsToClear.Add(l);
        }

        /// <summary>
        /// Used to both add a stack of cards to a new pile and delete this stack from its' old pile.
        ///     Also checks to see if placeholder is needed.
        /// </summary>
        /// <param name="stack">Card stack being moved.</param>
        /// <param name="newStackNum">New stack number.</param>
        /// <param name="oldStackNum">Old stack number.</param>
        private void updateColumn(ArrayList stack, int newStackNum, int oldStackNum)
        {
            if (newStackNum == 1)
            {
                t.updateCol1(stack, null);
            }
            else if (newStackNum == 2)
            {
                t.updateCol2(stack, null);
            }
            else if (newStackNum == 3)
            {
                t.updateCol3(stack, null);
            }
            else if (newStackNum == 4)
            {
                 t.updateCol4(stack, null);
            }
            else if (newStackNum == 5)
            {
                 t.updateCol5(stack, null);
            }
            else if (newStackNum == 6)
            {
                 t.updateCol6(stack, null);
            }
            else if (newStackNum == 7)
            {
                 t.updateCol7(stack, null);
            }

            if (oldStackNum == 1)
            {
                t.updateCol1(null, stack);
            }
            else if (oldStackNum == 2)
            {
                t.updateCol2(null, stack);
            }
            else if (oldStackNum == 3)
            {
                t.updateCol3(null, stack);
            }
            else if (oldStackNum == 4)
            {
                 t.updateCol4(null, stack);
            }
            else if (oldStackNum == 5)
            {
                 t.updateCol5(null, stack);
            }
            else if (oldStackNum == 6)
            {
                 t.updateCol6(null, stack);
            }
            else if (oldStackNum == 7)
            {
                 t.updateCol7(null, stack);
            }

            bool doIt = t.col1.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p1, true);
            doIt = t.col2.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p2, true);
            doIt = t.col3.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p3, true);
            doIt = t.col4.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p4, true);
            doIt = t.col5.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p5, true);
            doIt = t.col6.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p6, true);
            doIt = t.col7.needPlaceholder();
            if (doIt)
                makePlaceholder(t.p7, true);
        }

        /// <summary>
        /// This method is being called while program is running. Checks to see how many possible moves
        ///     can be made at any given moment, then displays this number on the bottom left of your screen.
        /// </summary>
        private void checkForMoves()
        {
            more = t.checkForMoreMoves();
            currentMoves.Text = "Number of Current Possible Moves: " + more;
            currentScore.Text = "Score: " + t.found.score;
            
            if (more == 0)
            {
                checkRemainingMoves.Enabled = false;
                if (t.checkSolved())
                {
                    //t.celebrate();   ---- Cool celebration screen.
                    //addToHighScores();   --- Someday, will have high score table.
                    
                    DialogBox db = new DialogBox(this, 500, 300, t.found.score, "You Win!");
                    db.Show();
                }
                else
                {
                    //addToHighScores(); --- Someday, will have high score table.
                    DialogBox db = new DialogBox(this, 500, 300, t.found.score, "Game Over!");
                    db.Show();
                }
            }
        }

        /// <summary>
        /// Scrunching is required if too many cards are placed on one stack. This method is called
        ///     after every clicking+dragging of a pile, and evaluates whether stacks need to
        ///     "scrunch" their cards to fit the entire pile on the screen. Will turn scrunching
        ///     "off" if not required.
        /// </summary>
        private void checkForScrunch()
        {
            t.col1.scrunchCol(t.col1.colUpSize() > 14);
            t.col2.scrunchCol(t.col2.colUpSize() > 13);
            t.col3.scrunchCol(t.col3.colUpSize() > 12);
            t.col4.scrunchCol(t.col4.colUpSize() > 12);
            t.col5.scrunchCol(t.col5.colUpSize() > 11);
            t.col6.scrunchCol(t.col6.colUpSize() > 11);
            t.col7.scrunchCol(t.col7.colUpSize() > 11);
        }

    }
}