using System;
using System.Windows.Forms;
using System.Collections;

namespace DragDrop
{
	/// <summary>
    /// Original class found at:
    /// http://pages.cpsc.ucalgary.ca/~carman/481/examples/csharp/dragdrop.html
    /// 
    /// Edited/tweaked by Stephanie Stubler
	/// </summary>
	public class MouseMove
	{

		// the current X and Y position
		public int curX;
		public int curY;

		// the last recorded X and Y position
		public int lastX;
		public int lastY;
        public int endX;
        public int endY;
        public ArrayList startX;
        public ArrayList startY;
        private ArrayList cardStack;

        public int upX, upY;

		// whether the control is currently moving
		private bool Moving;

		// a timer to control movement
		private Timer timer;

		public MouseMove(Timer tmr)
		{
			tmr.Enabled = false;
			tmr.Interval = 30;
			timer = tmr;

            startX = new ArrayList();
            startY = new ArrayList();
            cardStack = new ArrayList();
		}

		public void Begin(ArrayList c, int x, int y, int yInc)
		{
			Moving = true;

            startX.Clear();
            startY.Clear();
            cardStack.Clear();

            lastX = x;
            lastY = y;

            for (int i = 0; i < c.Count; i++)
            {
                cardStack.Add((Control) c[i]);
                startX.Add(((Control)c[i]).Location.X);
                startY.Add(((Control)c[i]).Location.Y);
                ((Control)c[i]).BringToFront();
            }
		}
		public void InProgress(int x, int y)
		{
            if (Moving)
            {
                timer.Enabled = true;
                curX = x - (int) lastX;
                curY = y - (int) lastY;
            }
		}
        /* Maybe I'll finish this function if I'm feeling extra cool.
        public void SimulateMovement(int startX2, int startY2, int endX2, int endY2)
        {
            int tempX, tempY, xInc, yInc;
            tempX = startX2;
            tempY = startY2;
            bool done = false;
            if (startX2 < endX2)
                xInc = 1;
            else
                xInc = -1;
            if (startY2 < endY2)
                yInc = 1;
            else
                yInc = -1;
            if (Moving)
            {
                timer.Enabled = true;
                while (!done)
                {
                    curX = tempX - startX2;
                    curY = tempY - startY2;
                    if (tempX2 != endX2)
                        tempX2 += xInc;
                    if (tempY2 != endY2)
                        tempY2 += yInc;
                    if ((tempX2 == endX2) && (tempY2 == endY2))
                        done = true;
                }
            }
        }
         * */
		public void EndIt()
		{
			Moving = false;
			timer.Enabled = false;
		}
        public void setEnd()
        {
            endX = ((int)((Control)cardStack[0]).Left + curX);
            endY = ((int)((Control)cardStack[0]).Top + curY);
        }
        public int getEndX()
        {
            return endX;
        }
        public int getEndY()
        {
            return endY;
        }
        public bool isMoving()
        {
            return Moving;
        }
        // Do I use this?
		public void ToNewPosition()
		{
			int x;
            int y;

			// set it to the new position
            for (int i = 0; i < cardStack.Count; i++)
            {
                x = (int) ((Control)cardStack[i]).Left + curX;
                y = (int) ((Control)cardStack[i]).Top + curY;
                ((Control)cardStack[i]).Location = new System.Drawing.Point(x, y);
            }
		}
        public void ToOldPosition()
        {
            for (int i = 0; i < cardStack.Count; i++)
            {
                ((Control)cardStack[i]).Location = new System.Drawing.Point((int) startX[i], (int) startY[i]);
            }
        }
        public void CardPeekUp(Control c, int incr)
        {
            upX = (int)c.Location.X;
            upY = (int)c.Location.Y;
            c.Location = new System.Drawing.Point(upX, upY - incr);
        }
        public void CardPeekDown(Control c)
        {
            c.Location = new System.Drawing.Point(upX, upY);
        }
        /// <summary>
        /// If drop is legal, snaps cards into correct position.
        /// </summary>
        /// <param name="x">Position of Card to Snap Under.</param>
        /// <param name="y">Position of Card to Snap Under.</param>
        /// <param name="yInc">Relative to card size.</param>
        public void SnapToNewPosition(int xOld, int yOld, int yInc)
        {
            int x, y;
            x = xOld;
            y = yOld;
            for (int i = 0; i < cardStack.Count; i++)
            {
                y += yInc;
                ((Control)cardStack[i]).Location = new System.Drawing.Point(x, y);
            }
        }
    }
}
