//------------------------------------------------------------------------------
//  Russian Solitaire! v.1.0
//      2006   Steph Stubler
//              University of Virginia
//              CS 216 - Programming & Data Representation [Prof. David Evans]
//
//              Table.cs
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Drawing;
using System.Threading;

namespace RussianSolitaire
{
    class Table
    {
        public Foundation found;
        public bool solved;
        public Column col1;
        public Column col2;
        public Column col3;
        public Column col4;
        public Column col5;
        public Column col6;
        public Column col7;
        public Card p1;
        public Card p2;
        public Card p3;
        public Card p4;
        public Card p5;
        public Card p6;
        public Card p7;
        public Card f1;
        public Card f2;
        public Card f3;
        public Card f4;
        public Deck deck;

        public Table(Bitmap back)
        {
            found = new Foundation();
            solved = false;
            deck = new Deck(back);
            deck.shuffle();
            deck.shuffle();
            deck.shuffle();
            deck.shuffle();
            deck.shuffle();
            int x = Constants.XBEGIN;
            col1 = new Column(deck.getCardFromDeck(0), 
                x, Constants.YBEGIN);
            p1 = new Card(1, x, Constants.YBEGIN, false, "place");
            x += Constants.XINC;
            col2 = new Column(deck.getCardFromDeck(1), deck.getCardFromDeck(2), 
                deck.getCardFromDeck(3), deck.getCardFromDeck(4), 
                deck.getCardFromDeck(5), deck.getCardFromDeck(6),
                x, Constants.YBEGIN);
            p2 = new Card(2, x, Constants.YBEGIN, false, "place");
            x += Constants.XINC;
            col3 = new Column(deck.getCardFromDeck(7), deck.getCardFromDeck(8), 
                deck.getCardFromDeck(9), deck.getCardFromDeck(10),
                deck.getCardFromDeck(11), deck.getCardFromDeck(12), 
                deck.getCardFromDeck(13),
                x, Constants.YBEGIN);
            p3 = new Card(3, x, Constants.YBEGIN, false, "place");
            x += Constants.XINC;
            col4 = new Column(deck.getCardFromDeck(14), deck.getCardFromDeck(15), 
                deck.getCardFromDeck(16), deck.getCardFromDeck(17),
                deck.getCardFromDeck(18), deck.getCardFromDeck(19), 
                deck.getCardFromDeck(20), deck.getCardFromDeck(21),
                x, Constants.YBEGIN);
            p4 = new Card(4, x, Constants.YBEGIN, false, "place");
            x += Constants.XINC;
            col5 = new Column(deck.getCardFromDeck(22), deck.getCardFromDeck(23), 
                deck.getCardFromDeck(24), deck.getCardFromDeck(25),
                deck.getCardFromDeck(26), deck.getCardFromDeck(27), 
                deck.getCardFromDeck(28), deck.getCardFromDeck(29),
                deck.getCardFromDeck(30),
                x, Constants.YBEGIN);
            p5 = new Card(5, x, Constants.YBEGIN, false, "place");
            x += Constants.XINC;
            col6 = new Column(deck.getCardFromDeck(31), deck.getCardFromDeck(32), 
                deck.getCardFromDeck(33), deck.getCardFromDeck(34),
                deck.getCardFromDeck(35), deck.getCardFromDeck(36), 
                deck.getCardFromDeck(37), deck.getCardFromDeck(38),
                deck.getCardFromDeck(39), deck.getCardFromDeck(40),
                x, Constants.YBEGIN);
            p6 = new Card(6, x, Constants.YBEGIN, false, "place");
            x += Constants.XINC;
            col7 = new Column(deck.getCardFromDeck(41), deck.getCardFromDeck(42), 
                deck.getCardFromDeck(43), deck.getCardFromDeck(44),
                deck.getCardFromDeck(45), deck.getCardFromDeck(46), 
                deck.getCardFromDeck(47), deck.getCardFromDeck(48),
                deck.getCardFromDeck(49), deck.getCardFromDeck(50),
                deck.getCardFromDeck(51),
                x, Constants.YBEGIN);
            p7 = new Card(7, x, Constants.YBEGIN, false, "place");

            f1 = new Card(8, Constants.FOUND1X, Constants.YFOUNDBEGIN, true, "f1");
            f2 = new Card(9, Constants.FOUND2X, Constants.YFOUNDBEGIN, true, "f2");
            f3 = new Card(10, Constants.FOUND3X, Constants.YFOUNDBEGIN, true, "f3");
            f4 = new Card(11, Constants.FOUND4X, Constants.YFOUNDBEGIN, true, "f4");

        }

        #region Update the Columns (Adds & Removes from Col ArrayLists)
        public void updateCol1(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col1.addStack(add);
            if (remove != null)
                col1.removeStack(remove);
        }

        public void updateCol2(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col2.addStack(add);
            if (remove != null)
                col2.removeStack(remove);
        }

        public void updateCol3(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col3.addStack(add);
            if (remove != null)
                col3.removeStack(remove);
        }

        public void updateCol4(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col4.addStack(add);
            if (remove != null)
                col4.removeStack(remove);
        }

        public void updateCol5(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col5.addStack(add);
            if (remove != null)
                col5.removeStack(remove);
        }

        public void updateCol6(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col6.addStack(add);
            if (remove != null)
                col6.removeStack(remove);
        }

        public void updateCol7(ArrayList add, ArrayList remove)
        {
            if (add != null)
                col7.addStack(add);
            if (remove != null)
                col7.removeStack(remove);
        }
#endregion

        /// <summary>
        /// Looks through every tableau pile and finds card with specified value and suit.
        ///     Will not search through "cant" pile, because it is not possible to move 
        ///     onto this pile.
        /// </summary>
        /// <param name="val">1-13</param>
        /// <param name="suit">Suit.</param>
        /// <param name="cant">Doesn't matter if card is in this pile.</param>
        /// <returns></returns>
        private bool searchStacks(int val, String suit, int cant)
        {
            bool found = false;
            Card c;
            for (int i = 0; i < col1.colSize(); i++)
            {
                c = ((Card)col1.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 1)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            for (int i = 0; i < col2.colSize(); i++)
            {
                c = ((Card)col2.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 2)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            for (int i = 0; i < col3.colSize(); i++)
            {
                c = ((Card)col3.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 3)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            for (int i = 0; i < col4.colSize(); i++)
            {
                c = ((Card)col4.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 4)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            for (int i = 0; i < col5.colSize(); i++)
            {
                c = ((Card)col5.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 5)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            for (int i = 0; i < col6.colSize(); i++)
            {
                c = ((Card)col6.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 6)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            for (int i = 0; i < col7.colSize(); i++)
            {
                c = ((Card)col7.getCard(i));
                if ((c.getValue() == val) && (c.getSuit().Equals(suit)) && (cant != 7)
                    && c.isFaceUp())
                {
                    if ((val == 13) && (i == 0))
                        found = false;
                    else
                        found = true;
                    break;
                }
            }
            return found;
        }

        /// <summary>
        /// Called while program is running.
        /// </summary>
        /// <returns>How many more possible moves at any given time.</returns>
        public int checkForMoreMoves()
        {
            int moves = 0;

            String suit;
            int value, needValue;
            Card c;
            #region Check Stack 1
            if (col1.colSize() != 0)
            {
                c = ((Card)col1.getCard(col1.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 1))
                    moves += 1;
            }
            else if ( (searchStacks(13, Constants.HEARTS, 1)) ||
                (searchStacks(13, Constants.DIAMONDS, 1)) ||
                (searchStacks(13, Constants.CLUBS, 1)) ||
                (searchStacks(13, Constants.SPADES, 1)) )
            {
                moves += 1;
            }
            #endregion

            #region Check Stack 2
            if (col2.colSize() != 0)
            {
                c = ((Card)col2.getCard(col2.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to Foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 2))
                    moves += 1;
            }
            else if ((searchStacks(13, Constants.HEARTS, 2)) ||
                (searchStacks(13, Constants.DIAMONDS, 2)) ||
                (searchStacks(13, Constants.CLUBS, 2)) ||
                (searchStacks(13, Constants.SPADES, 2)))
            {
                moves += 1;
            }
            #endregion

            #region Check Stack 3
            if (col3.colSize() != 0)
            {
                c = ((Card)col3.getCard(col3.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to Foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 3))
                    moves += 1;
            }
            else if ((searchStacks(13, Constants.HEARTS, 3)) ||
                (searchStacks(13, Constants.DIAMONDS, 3)) ||
                (searchStacks(13, Constants.CLUBS, 3)) ||
                (searchStacks(13, Constants.SPADES, 3)))
            {
                moves += 1;
            }
            #endregion

            #region Check Stack 4
            if (col4.colSize() != 0)
            {
                c = ((Card)col4.getCard(col4.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to Foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 4))
                    moves += 1;
            }
            else if ((searchStacks(13, Constants.HEARTS, 4)) ||
                (searchStacks(13, Constants.DIAMONDS, 4)) ||
                (searchStacks(13, Constants.CLUBS, 4)) ||
                (searchStacks(13, Constants.SPADES, 4)))
            {
                moves += 1;
            }
            #endregion

            #region Check Stack 5
            if (col5.colSize() != 0)
            {
                c = ((Card)col5.getCard(col5.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to Foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 5))
                    moves += 1;
            }
            else if ((searchStacks(13, Constants.HEARTS, 5)) ||
                (searchStacks(13, Constants.DIAMONDS, 5)) ||
                (searchStacks(13, Constants.CLUBS, 5)) ||
                (searchStacks(13, Constants.SPADES, 5)))
            {
                moves += 1;
            }
            #endregion

            #region Check Stack 6
            if (col6.colSize() != 0)
            {
                c = ((Card)col6.getCard(col6.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to Foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 6))
                    moves += 1;
            }
            else if ((searchStacks(13, Constants.HEARTS, 6)) ||
                (searchStacks(13, Constants.DIAMONDS, 6)) ||
                (searchStacks(13, Constants.CLUBS, 6)) ||
                (searchStacks(13, Constants.SPADES, 6)))
            {
                moves += 1;
            }
            #endregion

            #region Check Stack 7
            if (col7.colSize() != 0)
            {
                c = ((Card)col7.getCard(col7.colSize() - 1));
                suit = c.getSuit();
                value = c.getValue();
                needValue = value - 1;
                if (!(c.isFaceUp())) // Bottom Card can be flipped over, +1 move.
                    moves += 1;
                else if (needValue == 0) // Bottom Card is Ace, +1 move.
                    moves += 1;
                else if (found.isReady(c, false, false)) // Bottom card can be moved to Foundation.
                    moves += 1;
                // There exists a card not in the current stack that can be placed on this card.
                else if (searchStacks(needValue, suit, 7))
                    moves += 1;
            }
            else if ((searchStacks(13, Constants.HEARTS, 7)) ||
                (searchStacks(13, Constants.DIAMONDS, 7)) ||
                (searchStacks(13, Constants.CLUBS, 7)) ||
                (searchStacks(13, Constants.SPADES, 7)))
            {
                moves += 1;
            }
            #endregion

            return moves;
        }

        /// <summary>
        /// If all cards are in foundation, game is solved!
        /// </summary>
        /// <returns>True if Solved /false - not solved.</returns>
        public bool checkSolved()
        {
            if (found.checkHearts() && found.checkDiamonds() &&
                found.checkClubs() && found.checkSpades())
            {
                solved = true;
            }
            else
            {
                solved = false;
            }
            return solved;
        }

        /// <summary>
        /// Again, if I ever feel ambitious, make awesome end screen.
        /// </summary>
        public void celebrate()
        {
                // End screen
        }
    }
}
